/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/
#include <qcolordialog.h>
//Added by qt3to4:
//#include <Q3Frame>
#include <QCustomEvent>
#include "mainform.h"
#include "main.h"
#include "tartinisettingsdialog.h"
#include "gdata.h"
#include "audio_stream.h"
#include <QFileDialog>

TartiniSettingsDialog::TartiniSettingsDialog(QWidget *parent)
  : QDialog(parent, Qt::WDestructiveClose)
{
  setupUi(this);

  connect(okButton, SIGNAL(clicked()), this, SLOT(saveSettings()));
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

  connect( backgroundButton, SIGNAL( clicked() ), this, SLOT( getBackgroundColor() ) );
  connect( shading1Button, SIGNAL( clicked() ), this, SLOT( getShading1Color() ) );
  connect( shading2Button, SIGNAL( clicked() ), this, SLOT( getShading2Color() ) );
  connect( noteRangeChoice, SIGNAL( activated(int) ), this, SLOT( onNoteRangeChoice(int) ) );
  connect( resetDefaultsButton, SIGNAL( clicked() ), this, SLOT( resetDefaults() ) );
  connect( temperament, SIGNAL( activated(int) ),
	   this, SLOT( onTemperamentChange(int) ) );

  init();
}

void TartiniSettingsDialog::loadSetting(QObject *obj, const QString &group)
{
  QString key = obj->name();
  QString fullKey = group + "/" + key;

  if(obj->isA("QGroupBox")) {
    //Iterate over the groupBox's children
    const QList<QObject*> &widgets = obj->children();
    for(QList<QObject*>::const_iterator it = widgets.begin(); it < widgets.end(); ++it) {
      loadSetting(*it, group);
    }
#ifdef SRT
  } else if(obj->isA("QLineEdit")) {
    //((QLineEdit*)obj)->setText((gdata->settings).getString(group, key));
    ((QLineEdit*)obj)->setText(gdata->qsettings->value(fullKey).toString());
#endif
  } else if(obj->isA("QComboBox")) {
    ((QComboBox*)obj)->setCurrentText(gdata->qsettings->value(fullKey).toString());
  } else if(obj->isA("QPushButton") && ((QPushButton*)obj)->isToggleButton()) {
    ((QPushButton*)obj)->setOn(gdata->qsettings->value(fullKey).toBool());
  } else if(obj->isA("QCheckBox")) {
    ((QCheckBox*)obj)->setChecked(gdata->qsettings->value(fullKey).toBool());
  } else if(obj->isA("QSpinBox")) {
    ((QSpinBox*)obj)->setValue(gdata->qsettings->value(fullKey).toInt());
  } else if(obj->isA("QFrame")) {
    QColor color;
    color.setNamedColor(gdata->qsettings->value(fullKey).toString());
    ((QFrame*)obj)->setPaletteBackgroundColor(color);
  }
}

void TartiniSettingsDialog::init()
{
  // Go through all the categories on the left, and insert all the preferences we can into the fields.
  // Combo boxes must be done separately.
  soundInput->clear();
  soundInput->insertStringList(AudioStream::getInputDeviceNames());
  
  QString group;
  //Iterate over all groups
  for (int i = 0; i < tabWidget->count(); i++) {
    //Iterate over all widgets in the current group and load their settings
    group = tabWidget->tabText(i);
    const QList<QObject*> &widgets = tabWidget->widget(i)->children();
    for(QList<QObject*>::const_iterator it=widgets.begin(); it < widgets.end(); ++it) {
      loadSetting(*it, group);
    }
  }

  checkAnalysisEnabled();
  setCustomTempEnabled(temperament->currentItem() == 2);
}

QString TartiniSettingsDialog::getPath(const QString initialPath)
{
  //return Q3FileDialog::getExistingDirectory(initialPath, this, "getDirectory", "Choose a directory", true, false);
  return QFileDialog::getExistingDirectory(this, "Choose a directory", initialPath);
}


void TartiniSettingsDialog::getBackgroundColor()
{
  QColor color = QColorDialog::getColor(theBackgroundColor->paletteBackgroundColor());
  if(color.isValid()) {
    theBackgroundColor->setPaletteBackgroundColor(color);
  }
}

void TartiniSettingsDialog::getShading1Color()
{
  QColor color = QColorDialog::getColor(shading1Color->paletteBackgroundColor());
  if(color.isValid()) {
    shading1Color->setPaletteBackgroundColor(color);
  }
}

void TartiniSettingsDialog::getShading2Color()
{
  QColor color = QColorDialog::getColor(shading2Color->paletteBackgroundColor());
  if(color.isValid()) {
    shading2Color->setPaletteBackgroundColor(color);
  }
}

void TartiniSettingsDialog::saveSetting(QObject *obj, const QString group)
{
  QString key = obj->name();
  QString fullKey = group + "/" + key;

  if(obj->isA("QGroupBox")) {
    //Iterate over the groupBox's children
    const QList<QObject*> &widgets = obj->children();
    for(QList<QObject*>::const_iterator it=widgets.begin(); it < widgets.end(); ++it) {
      saveSetting(*it, group);
    }
#ifdef SRT
  } else if(obj->isA("QLineEdit")) {
    gdata->qsettings->setValue(fullKey, ((QLineEdit*)obj)->text());
#endif
  } else if(obj->isA("QComboBox")) {
    gdata->qsettings->setValue(fullKey, ((QComboBox*)obj)->currentText());
  } else if(obj->isA("QPushButton") && ((QPushButton*)obj)->isToggleButton()) {
    gdata->qsettings->setValue(fullKey, ((QPushButton*)obj)->isOn());
  } else if(obj->isA("QCheckBox")) {
    gdata->qsettings->setValue(fullKey, ((QCheckBox*)obj)->isChecked());
  } else if(obj->isA("QSpinBox")) {
    gdata->qsettings->setValue(fullKey, ((QSpinBox*)obj)->value());
  } else if(obj->isA("QFrame")) {
    gdata->qsettings->setValue(fullKey, ((QFrame*)obj)->paletteBackgroundColor().name());
  }
}

void TartiniSettingsDialog::saveSettings()
{
  // Go through all the categories on the left, and save all the preferences we can from the fields.
  // Combo boxes must be done separately.
  QString group;
  //Iterate over all the groups
  for(int i = 0; i < tabWidget->count(); i++) {
    //Iterate over all widgets in the current group and save their settings
    group = tabWidget->tabText(i);
    const QList<QObject*> &widgets = tabWidget->widget(i)->children();
    for(QList<QObject*>::const_iterator it=widgets.begin(); it < widgets.end(); ++it) {
      saveSetting(*it, group);
    }
  }

  //
  //  If we've got a Special temperament in effect, then save it.
  //
  switch(temperament->currentIndex()) {
  case 2:
    writeSpecialTemperament("Special1");
    break;
  case 3:
    writeSpecialTemperament("Special2");
    break;
  case 4:
    writeSpecialTemperament("Special3");
    break;
  };
  
  gdata->qsettings->sync();
  QApplication::postEvent(mainWindow, new QCustomEvent(SETTINGS_CHANGED));
  TartiniSettingsDialog::accept();
}

void TartiniSettingsDialog::checkAnalysisEnabled()
{
  QComboBox *noteRangeChoice = tabWidget->widget(0)->findChild<QComboBox *>("noteRangeChoice");
  myassert(noteRangeChoice);
  
  int choice = noteRangeChoice->currentItem();
  
  QGroupBox *bufferSizeGroupBox = tabWidget->widget(2)->findChild<QGroupBox*>("bufferSizeGroupBox");
  myassert(bufferSizeGroupBox);
  QGroupBox *stepSizeGroupBox = tabWidget->widget(2)->findChild<QGroupBox*>("stepSizeGroupBox");
  myassert(stepSizeGroupBox);
  
  if(choice == 0) {
    bufferSizeGroupBox->setEnabled(true);
    stepSizeGroupBox->setEnabled(true);
  } else {
    bufferSizeGroupBox->setEnabled(false);
    stepSizeGroupBox->setEnabled(false);
  }
}

void TartiniSettingsDialog::setTemperament(const int *t)
{
  customIntonationC->setValue(t[0]);
  customIntonationCS->setValue(t[1]);
  customIntonationD->setValue(t[2]);
  customIntonationDS->setValue(t[3]);
  customIntonationE->setValue(t[4]);
  customIntonationF->setValue(t[5]);
  customIntonationFS->setValue(t[6]);
  customIntonationG->setValue(t[7]);
  customIntonationGS->setValue(t[8]);
  customIntonationA->setValue(t[9]);
  customIntonationAS->setValue(t[10]);
  customIntonationB->setValue(t[11]);
};

void TartiniSettingsDialog::setCustomTempEnabled(bool flag)
{
  customIntonationC->setEnabled(flag);
  customIntonationCS->setEnabled(flag);
  customIntonationD->setEnabled(flag);
  customIntonationDS->setEnabled(flag);
  customIntonationE->setEnabled(flag);
  customIntonationF->setEnabled(flag);
  customIntonationFS->setEnabled(flag);
  customIntonationG->setEnabled(flag);
  customIntonationGS->setEnabled(flag);
  customIntonationA->setEnabled(flag);
  customIntonationAS->setEnabled(flag);
  customIntonationB->setEnabled(flag);
}

const int equaltemp[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
const int justdtemp[12] = {-4, -12, 0, 12, 4, 16, -14, -2, -10, 2, 14, -16};

void TartiniSettingsDialog::readSpecialTemperament(char *name)
{
  char tmp[256];

  int specialtemp[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
  for (int i = 0; i < 12; ++i) {
    sprintf(tmp, "%s/%d", name, i);
    specialtemp[i] = gdata->qsettings->value(tmp).toInt();
  }
  setTemperament(specialtemp);
};

void TartiniSettingsDialog::writeSpecialTemperament(char *name)
{
  char tmp[256];

  sprintf(tmp, "%s/%d", name, 0);
  gdata->qsettings->setValue(tmp, customIntonationC->value());

  sprintf(tmp, "%s/%d", name, 1);
  gdata->qsettings->setValue(tmp, customIntonationCS->value());

  sprintf(tmp, "%s/%d", name, 2);
  gdata->qsettings->setValue(tmp, customIntonationD->value());

  sprintf(tmp, "%s/%d", name, 3);
  gdata->qsettings->setValue(tmp, customIntonationDS->value());

  sprintf(tmp, "%s/%d", name, 4);
  gdata->qsettings->setValue(tmp, customIntonationE->value());

  sprintf(tmp, "%s/%d", name, 5);
  gdata->qsettings->setValue(tmp, customIntonationF->value());

  sprintf(tmp, "%s/%d", name, 6);
  gdata->qsettings->setValue(tmp, customIntonationFS->value());

  sprintf(tmp, "%s/%d", name, 7);
  gdata->qsettings->setValue(tmp, customIntonationG->value());

  sprintf(tmp, "%s/%d", name, 8);
  gdata->qsettings->setValue(tmp, customIntonationGS->value());

  sprintf(tmp, "%s/%d", name, 9);
  gdata->qsettings->setValue(tmp, customIntonationA->value());

  sprintf(tmp, "%s/%d", name, 10);
  gdata->qsettings->setValue(tmp, customIntonationAS->value());

  sprintf(tmp, "%s/%d", name, 11);
  gdata->qsettings->setValue(tmp, customIntonationB->value());

  gdata->qsettings->sync();
};

void TartiniSettingsDialog::onTemperamentChange(int choice)
{
  switch(choice) {
  case 0:
    setTemperament(equaltemp);
    setCustomTempEnabled(false);
    break;
  case 1:
    setTemperament(justdtemp);
    setCustomTempEnabled(false);
    break;
  case 2:
    // Special 1
    readSpecialTemperament("Special1");
    setCustomTempEnabled(true);
    break;
  case 3:
    // Special 2
    readSpecialTemperament("Special2");
    setCustomTempEnabled(true);
    break;
  case 4:
    // Special 3
    readSpecialTemperament("Special3");
    setCustomTempEnabled(true);
    break;
  case 5:
    setCustomTempEnabled(true);
    break;
  };
};

void TartiniSettingsDialog::onNoteRangeChoice(int choice)
{
  QSpinBox *bufferSizeValue = tabWidget->widget(2)->findChild<QSpinBox*>("bufferSizeValue");
  myassert(bufferSizeValue);
  QComboBox *bufferSizeUnit = tabWidget->widget(2)->findChild<QComboBox*>("bufferSizeUnit");
  myassert(bufferSizeUnit);
  QCheckBox *bufferSizeRound = tabWidget->widget(2)->findChild<QCheckBox*>("bufferSizeRound");
  myassert(bufferSizeRound);
  QSpinBox *stepSizeValue = tabWidget->widget(2)->findChild<QSpinBox*>("stepSizeValue");
  myassert(stepSizeValue);
  QComboBox *stepSizeUnit = tabWidget->widget(2)->findChild<QComboBox*>("stepSizeUnit");
  myassert(stepSizeUnit);
  QCheckBox *stepSizeRound = tabWidget->widget(2)->findChild<QCheckBox*>("stepSizeRound");
  myassert(stepSizeRound);

  switch(choice) {
    case 1:
        bufferSizeValue->setValue(96);
        stepSizeValue->setValue(48);
      break;
    case 2:
        bufferSizeValue->setValue(48);
        stepSizeValue->setValue(24);
      break;
    case 3:
        bufferSizeValue->setValue(24);
        stepSizeValue->setValue(12);
      break;
    case 4:
        bufferSizeValue->setValue(12);
        stepSizeValue->setValue(6);
      break;
  }
  if(choice > 0) {
    bufferSizeUnit->setCurrentText("milli-seconds");
    stepSizeUnit->setCurrentText("milli-seconds");
    bufferSizeRound->setChecked(true);
    stepSizeRound->setChecked(true);
  }
  checkAnalysisEnabled();
}

#define SetIfMissing(key, value) \
  if(!s->contains(key)) s->setValue(key, value)

void TartiniSettingsDialog::setUnknownsToDefault(QSettings *s)
{
  SetIfMissing("General/bindOpenSaveFolders", true);
  SetIfMissing("General/noteRangeChoice", "Notes F3 and higher - Flute, Piccolo ...");

  SetIfMissing("View/autoFollow", false);
  SetIfMissing("View/backgroundShading", true);
  SetIfMissing("View/freqA", 440);

  SetIfMissing("Sound/soundInput", "Default");
  SetIfMissing("Sound/numberOfBuffers", 4);
  SetIfMissing("Sound/numberOfChannels", "Mono");
  SetIfMissing("Sound/sampleRate", 44100);
  SetIfMissing("Sound/bitsPerSample", 16);
  SetIfMissing("Sound/muteOutput", true);

  SetIfMissing("Analysis/bufferSizeValue", 12);
  SetIfMissing("Analysis/bufferSizeUnit", "milli-seconds");
  SetIfMissing("Analysis/bufferSizeRound", true);
  SetIfMissing("Analysis/stepSizeValue", 6);
  SetIfMissing("Analysis/stepSizeUnit", "milli-seconds");
  SetIfMissing("Analysis/stepSizeRound", true);
  SetIfMissing("Analysis/doingHarmonicAnalysis", true);
  SetIfMissing("Analysis/doingFreqAnalysis", true);
  SetIfMissing("Analysis/doingEqualLoudness", true);
  SetIfMissing("Analysis/doingAutoNoiseFloor", true);
  SetIfMissing("Analysis/doingDetailedPitch", true);
  SetIfMissing("Analysis/analysisType", "MPM");
  SetIfMissing("Analysis/thresholdValue", 93);

  SetIfMissing("Display/fastUpdateSpeed", 75);
  SetIfMissing("Display/slowUpdateSpeed", 150);
  SetIfMissing("Display/theBackgroundColor", "#BBCDE2");
  SetIfMissing("Display/shading1Color", "#BBCDEF");
  SetIfMissing("Display/shading2Color", "#CBCDE2");
  SetIfMissing("Display/displayBackgroundShading", true);
#ifdef SRT
  SetIfMissing("Display/useTopLevelWidgets", false);
#endif

  SetIfMissing("Dialogs/rememberOpenFolder", true);
  SetIfMissing("Dialogs/openFilesFolder", QDir::convertSeparators(QDir::currentDirPath()));
  SetIfMissing("Dialogs/rememberSaveFolder", true);
  SetIfMissing("Dialogs/saveFilesFolder", QDir::convertSeparators(QDir::currentDirPath()));
  SetIfMissing("Dialogs/appendWav", true);

  SetIfMissing("Advanced/showMeanVarianceBars", false);
  SetIfMissing("Advanced/savingMode", "Ask when closing unsaved files (normal)");
  SetIfMissing("Advanced/vibratoSineStyle", false);
  SetIfMissing("Advanced/mouseWheelZooms", false);

  SetIfMissing("RTTA/volume_threshold", -100);
  SetIfMissing("RTTA/clarity_threshold", 25);
  SetIfMissing("RTTA/yellow_limit", 20);
  SetIfMissing("RTTA/red_limit", 30);
  SetIfMissing("RTTA/samples_threshold", 2);
  SetIfMissing("RTTA/printAnnotation", false);
  SetIfMissing("RTTA/summaryAnnotation", false);
  SetIfMissing("RTTA/basePitchA", 440);
  SetIfMissing("RTTA/temperament", "Equal");
  SetIfMissing("RTTA/customIntonationC", 0);
  SetIfMissing("RTTA/customIntonationCS", 0);
  SetIfMissing("RTTA/customIntonationD", 0);
  SetIfMissing("RTTA/customIntonationDS", 0);
  SetIfMissing("RTTA/customIntonationE", 0);
  SetIfMissing("RTTA/customIntonationF", 0);
  SetIfMissing("RTTA/customIntonationFS", 0);
  SetIfMissing("RTTA/customIntonationG", 0);
  SetIfMissing("RTTA/customIntonationGS", 0);
  SetIfMissing("RTTA/customIntonationA", 0);
  SetIfMissing("RTTA/customIntonationAS", 0);
  SetIfMissing("RTTA/customIntonationB", 0);
}

void TartiniSettingsDialog::resetDefaults()
{
  gdata->qsettings->clear();
  setUnknownsToDefault(gdata->qsettings);
  gdata->qsettings->sync();
  init();
}
