//
//  Thu Jun 26 21:48:31 2008 -- Scott R. Turner
//  RTTAVIEW
#include <qpixmap.h>
#include <qpainter.h>
//Added by qt3to4:
#include <QPaintEvent>
#include <Qt>

#include "rttawidget.h"
#include "gdata.h"
#include "channel.h"
#include "analysisdata.h"
#include "useful.h"

RttaWidget::RttaWidget(QWidget *parent)
  : DrawWidget(parent)
{
  textY = 0;
  //make any connections
  connect(gdata->view, SIGNAL(onSlowUpdate(double)), this, SLOT(update()));
}

RttaWidget::~RttaWidget()
{
}

void RttaWidget::printString(const QString &s) {
  p.setBackgroundMode(Qt::TransparentMode);
  p.setPen("black");
  textY += fontHeight;
  p.drawText(0, textY, s);
}

void RttaWidget::printString(const QString &s, const char *col) {
  p.setBackgroundMode(Qt::OpaqueMode);
  p.setBackground(QBrush(col));
  textY += fontHeight;
  p.drawText(0, textY, s);
}

void RttaWidget::paintEvent( QPaintEvent * )
{
  // Channel *ch = gdata->getActiveChannel();
  RTTAinfo *rInfo = gdata->rtta_data->rttaInfo;
  int total_samples = gdata->rtta_data->total_samples;
  QFont font("Courier", 10, QFont::Bold);
  QString s;

  beginDrawing();

  // Drawing code goes here
  p.setFont(font);
  QFontMetrics fm = p.fontMetrics();
  fontHeight = fm.height() + 5;

  textY = 0; //move text position back to the top
  printString(s.sprintf("%6s %8s %8s", "Note", "Samples", "Cents"));

  if (total_samples > 0) {
    int cutoff = (int) total_samples * gdata->getSamplesThreshold();
    int cents;
    int yellow_limit = gdata->getYellowLimit();
    int red_limit = gdata->getRedLimit();

    for(int i=0; i<RTTA_NOTE_RANGE; i++) {
      if (rInfo[i].num_samples > cutoff) {
	cents = (int) (((rInfo[i].total/rInfo[i].num_samples) - rInfo[i].start_val - 0.50) * 100);
	if (cents > red_limit || cents < -red_limit) {
	  printString(s.sprintf("%5s: %8d %8d         ", rInfo[i].note_name,
				rInfo[i].num_samples, cents), "red");
	} else if (cents > yellow_limit || cents < -yellow_limit) {
	  printString(s.sprintf("%5s: %8d %8d         ", rInfo[i].note_name,
				rInfo[i].num_samples, cents), "yellow");
	} else {
	  printString(s.sprintf("%5s: %8d %8d         ", rInfo[i].note_name, rInfo[i].num_samples, cents));
	};
      }
    }
  }

  endDrawing();
}
