/***************************************************************************
                          audio_stream.cpp  -  windows audio routines
                             -------------------
    begin                : 2003
    copyright            : (C) 2003-2005 by Philip McLeod
    email                : pmcleod@cs.otago.ac.nz
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   Please read LICENSE.txt for details.
 ***************************************************************************/
#include "audio_stream.h"

#include <map>
#include <qstringlist.h>

#include "myassert.h"
#include "gdata.h"

std::map<QString, uint> AudioStream::inputDevices;
std::map<QString, uint> AudioStream::outputDevices;

#define MAX_RECORDING_BUFFERS   16

int CalcHigherLogTwo(int n)
{
    int log2 = 0;
    while( (1<<log2) < n ) log2++;
    return log2;
}

int AudioBuffer::wait_for_buffer()
{
	int v;
	bufs_mutex.lock();
	v = free_bufs;
	bufs_mutex.unlock();
	//printf("free_bufs=%d\n", v);

	//don't know why but this seems to work better than waiting for the bufferEvent
	while(v == 0) {
		Sleep(1);
		bufs_mutex.lock();
		v = free_bufs;
		bufs_mutex.unlock();
	}
	//if(v == 0) WaitForSingleObject(bufferEvent, INFINITE);
	//else ResetEvent(bufferEvent);
	/*
	if(v == 0) {
		WaitForSingleObject(bufferEvent, INFINITE);
	} else {
		while(WaitForSingleObject(bufferEvent, 0) == WAIT_OBJECT_0) { }
	}
	*/
    return v;
}


void CALLBACK waveOutCallback(HWAVEOUT hwo, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	AudioStream *as = (AudioStream *)dwInstance;
	if(uMsg == WOM_DONE) {
		as->out.bufs_mutex.lock();
		as->out.free_bufs++;
		as->out.bufs_mutex.unlock();
		SetEvent(as->out.bufferEvent);
	}
}

void CALLBACK waveInCallback(HWAVEIN hwi, UINT uMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2)
{
	AudioStream *as = (AudioStream *)dwInstance;
	if(uMsg == WIM_DATA) {
		//printf("%d, %d, %d\n", dwParam1, dwParam2, ((WAVEHDR *)dwParam1)->dwFlags & WHDR_DONE);
		as->in.bufs_mutex.lock();
		as->in.free_bufs++;
		as->in.bufs_mutex.unlock();
		SetEvent(as->in.bufferEvent);
	//} else if(uMsg == WIM_OPEN) {
		//printf("WIM_OPEN\n");
	//} else {
		//printf("WIM_CLOSE\n");
	}
}

AudioStream::AudioStream()
{
  in_device_num = out_device_num = 0;
  //buffer_size = 1024;
  been_init = false;
}

AudioStream::~AudioStream()
{
	close();
}

void AudioStream::close()
{
	if(!been_init) return;
	if((mode & F_WRITE)) {
		waveOutReset(hwo);
		for(int j=0; j<out.num_bufs; j++) {
 			waveOutUnprepareHeader(hwo, &out.wavehdr[j], sizeof(WAVEHDR));
		}
		waveOutClose(hwo);
		free(out.wavehdr);
		CloseHandle(out.bufferEvent);
	} //else
	if((mode & F_READ)) {
		waveInReset(hwi);
		for(int j=0; j<in.num_bufs; j++) {
 			waveInUnprepareHeader(hwi, &in.wavehdr[j], sizeof(WAVEHDR));
		}
		waveInClose(hwi);
		// SRT -- MEMLEAK
	        for(int j=0; j<in.num_bufs; j++) {
		  free(in.wavehdr[j].lpData);
		};
		free(in.wavehdr);
		CloseHandle(in.bufferEvent);
	}
	been_init = false;
}

void AudioStream::signal1()
{
}

void AudioStream::print_state()
{
}

void AudioStream::set_params()
{
}

int AudioStream::open(int mode_, int freq_, int channels_, int bits_, int buffer_size_)
{
  mode = mode_;
  freq = freq_;
  channels = channels_;
  bits = bits_;
  //buffer_size = buffer_size_;

  int j;

  WAVEFORMATEX wfx;
  wfx.wFormatTag = WAVE_FORMAT_PCM;
  wfx.nChannels = (WORD) channels;
  wfx.nSamplesPerSec = (DWORD) freq;
  wfx.nAvgBytesPerSec = (DWORD)(frame_size() * freq);
  wfx.nBlockAlign = (WORD)frame_size();
  wfx.wBitsPerSample = (WORD)bits;
  wfx.cbSize = 0;

  if((mode & F_WRITE)) {
	out.init(buffer_size_, 8, 8);
	//out->num_bufs = 8;
    //bufferEvent = CreateEvent( NULL, false, false, NULL );
    out.bufferEvent = CreateEvent( NULL, false, false, NULL );
	if(out.bufferEvent == NULL) {
		fprintf(stderr, "AudioStream::open. Error creating event\n");
		return(1);
	}
	//waveOutOpen(&hwo, WAVE_MAPPER, &wfx, 0, 0, CALLBACK_NULL);
	//if(waveOutOpen(&hwo, WAVE_MAPPER, &wfx, (DWORD)bufferEvent, (DWORD)NULL, CALLBACK_EVENT) != MMSYSERR_NOERROR) {
	//uint deviceNumber = getOutputDeviceNumber(gdata->settings.getString("Sound", "soundOutput"));
	out_device_num = gdata->qsettings->value("Sound/soundOutput", 0).toInt();
	if(waveOutOpen(&hwo, out_device_num, &wfx, (DWORD)waveOutCallback, (DWORD)this, CALLBACK_FUNCTION) != MMSYSERR_NOERROR) {
		fprintf(stderr, "AudioStream::open. Error opening waveOut\n");
		return(1);
	}
	if((int)wfx.nChannels != channels) { fprintf(stderr, "Changed channels from %d to %d\n", (int)wfx.nChannels, channels); channels = (int)wfx.nChannels; }
	if((int)wfx.nSamplesPerSec != freq) { fprintf(stderr, "Changed freq from %d to %d\n", (int)wfx.nSamplesPerSec, freq); freq = (int)wfx.nChannels; }
	if((int)wfx.wBitsPerSample != bits) { fprintf(stderr, "Changed bits from %d to %d\n", (int)wfx.wBitsPerSample, bits); bits = (int)wfx.wBitsPerSample; }
	/*if(WaitForSingleObject(bufferEvent, 0) == WAIT_OBJECT_0) {
		fprintf(gfile, "openOut event\n");
	}*/
	out.wavehdr = (WAVEHDR *)calloc(out.num_bufs, sizeof(WAVEHDR));
	for(j=0; j<out.num_bufs; j++) {
		out.wavehdr[j].lpData = (char *)malloc(out.buffer_size * frame_size());
		out.wavehdr[j].dwBufferLength = out.buffer_size * frame_size();
		out.wavehdr[j].dwUser = j;
		waveOutPrepareHeader(hwo, &out.wavehdr[j], sizeof(WAVEHDR));
	}
	//cur_buf = 0;
	//buf_pos = 0;
	//free_bufs = num_bufs;
  } //else

  if((mode & F_READ)) {
    in.init(buffer_size_, MAX_RECORDING_BUFFERS, 0);
    //num_bufs = MAX_RECORDING_BUFFERS;
    in.bufferEvent = CreateEvent( NULL, false, false, NULL );
	if(in.bufferEvent == NULL) {
		fprintf(stderr, "AudioStream::open. Error creating event\n");
		return(1);
	}
	//if(waveInOpen(&hwi, WAVE_MAPPER, &wfx, (DWORD)bufferEvent, (DWORD)NULL, CALLBACK_EVENT) != MMSYSERR_NOERROR) {
	//uint deviceNumber = getInputDeviceNumber(gdata->settings.getString("Sound", "soundInput"));
	in_device_num = gdata->qsettings->value("Sound/soundInput", 0).toInt();
	if(waveInOpen(&hwi, in_device_num, &wfx, (DWORD)waveInCallback, (DWORD)this, CALLBACK_FUNCTION) != MMSYSERR_NOERROR) {
		fprintf(stderr, "AudioStream::open. Error opening waveOut\n");
		return(1);
	}
	if((int)wfx.nChannels != channels) { fprintf(stderr, "Changed channels from %d to %d\n", (int)wfx.nChannels, channels); channels = (int)wfx.nChannels; }
	if((int)wfx.nSamplesPerSec != freq) { fprintf(stderr, "Changed freq from %d to %d\n", (int)wfx.nSamplesPerSec, freq); freq = (int)wfx.nChannels; }
	if((int)wfx.wBitsPerSample != bits) { fprintf(stderr, "Changed bits from %d to %d\n", (int)wfx.wBitsPerSample, bits); bits = (int)wfx.wBitsPerSample; }
	/*if(WaitForSingleObject(bufferEvent, 0) == WAIT_OBJECT_0) {
		fprintf(gfile, "openIn event\n");
	}*/
	in.wavehdr = (WAVEHDR *)calloc(in.num_bufs, sizeof(WAVEHDR));
	for(j=0; j<in.num_bufs; j++) {
		in.wavehdr[j].lpData = (char *)malloc(in.buffer_size * frame_size());
		in.wavehdr[j].dwBufferLength = in.buffer_size * frame_size();
		in.wavehdr[j].dwUser = j;
		waveInPrepareHeader(hwi, &in.wavehdr[j], sizeof(WAVEHDR));
	}
	//cur_buf = 0;
	//buf_pos = 0;
	//free_bufs = 0;
	for(j=0; j<in.num_bufs; j++) {
		waveInAddBuffer(hwi, &in.wavehdr[j], sizeof(WAVEHDR));
	}
	waveInStart(hwi);
  } //else {
	//return -1;
  //}
  been_init = true;
  return 0;
}

/**
 * Returns a QStringList of available output devices
 **/
QStringList AudioStream::getOutputDeviceNames()
{
	QStringList toReturn;
	toReturn += "Default Output Device";

	AudioStream::outputDevices.clear();
	AudioStream::outputDevices["Default Output Device"] = WAVE_MAPPER; // The default device

	WAVEOUTCAPS wavCaps;
	uint devices = waveOutGetNumDevs();
	// szPname is a Unicode encoded string, so we need to put it into a QString
	// as they support Unicode (rather than char*).
	// See http://www.codeproject.com/string/cppstringguide1.asp
	QString temp;
	for (uint i = 0; i < devices; i++) {
		if (!waveOutGetDevCaps(i, &wavCaps, sizeof(WAVEOUTCAPS))) {
			//toReturn += temp.setUnicodeCodes(wavCaps.szPname, lstrlenW(wavCaps.szPname));
			temp = QString(wavCaps.szPname);
			toReturn += temp;
			AudioStream::outputDevices[temp] = i;
		}
	}
	
	
	return toReturn;
}

uint AudioStream::getInputDeviceNumber(QString toFind)
{
	if (AudioStream::inputDevices.size() == 0) {
		AudioStream::getInputDeviceNames(); // Make it generate the input devices
	}

	std::map<QString, uint>::const_iterator iter = AudioStream::inputDevices.find(toFind);
	if (iter != AudioStream::inputDevices.end()) {
		return iter->second;
	} else {
		return WAVE_MAPPER;
	}
}

uint AudioStream::getOutputDeviceNumber(QString toFind)
{
	if (AudioStream::outputDevices.size() == 0) {
		AudioStream::getOutputDeviceNames(); // Make it generate the output devices
	}
	std::map<QString, uint>::const_iterator iter = AudioStream::outputDevices.find(toFind);
	if (iter != AudioStream::outputDevices.end()) {
		return iter->second;
	} else {
		return WAVE_MAPPER;
	}
}


/**
 * Returns a QStringList of available input devices
 **/
QStringList AudioStream::getInputDeviceNames()
{
	QStringList toReturn;
	toReturn += "Default Input Device";

	AudioStream::inputDevices.clear();
	AudioStream::inputDevices["Default Input Device"] = WAVE_MAPPER; // The default device

	WAVEINCAPS wavCaps;
	uint devices = waveInGetNumDevs();
	// szPname is a Unicode encoded string, so we need to put it into a QString
	// as they support Unicode (rather than char*).
	// See http://www.codeproject.com/string/cppstringguide1.asp
	QString temp;
	for (uint i = 0; i < devices; i++) {
		if (!waveInGetDevCaps(i, &wavCaps, sizeof(WAVEINCAPS))) {
			//toReturn += temp.setUnicodeCodes(wavCaps.szPname, lstrlenW(wavCaps.szPname));
			temp = QString(wavCaps.szPname);
			toReturn += temp;
			AudioStream::inputDevices[temp] = i;
		}
	}
	
	
	return toReturn;
}

long AudioStream::read_bytes(void *data, long length)
{
	/*if(free_bufs == 0) {
		WaitForSingleObject(bufferEvent, INFINITE);
		free_bufs++;
	} else {
		if(WaitForSingleObject(bufferEvent, 0) == WAIT_OBJECT_0) {
			free_bufs++;
		}
	}*/
	int temp = in.wait_for_buffer();
	//printf("free_bufs=%d\n", temp);
	if(temp == MAX_RECORDING_BUFFERS) fprintf(stderr, "read_bytes: Out of buffers. %d used\n", temp);


	int remaining = (in.buffer_size * frame_size()) - in.buf_pos;
	//printf("%d, %d\n", remaining, length);
	if(remaining > length) {
		memcpy(data, in.wavehdr[in.cur_buf].lpData+in.buf_pos, length);
		in.buf_pos += length;
	} else
	if(remaining == length) {
		memcpy(data, in.wavehdr[in.cur_buf].lpData+in.buf_pos, length);
		waveInAddBuffer(hwi, &in.wavehdr[in.cur_buf], sizeof(WAVEHDR));
		in.bufs_mutex.lock();
		--in.free_bufs;
		in.bufs_mutex.unlock();
		in.buf_pos = 0;
		in.cur_buf = (in.cur_buf+1) % in.num_bufs;
	} else {
		memcpy(data, in.wavehdr[in.cur_buf].lpData+in.buf_pos, remaining);
		waveInAddBuffer(hwi, &in.wavehdr[in.cur_buf], sizeof(WAVEHDR));
		in.bufs_mutex.lock();
		--in.free_bufs;
		in.bufs_mutex.unlock();
		in.buf_pos = 0;
		in.cur_buf = (in.cur_buf+1) % in.num_bufs;
		read_bytes(((char*)data)+remaining, length-remaining);
	}
	return length;//read(device_no, data, length);
}

long AudioStream::read_frames(void *data, long length)
{
  return 0;//read(device_no, data, length * frame_size()) / frame_size();
}

long AudioStream::write_bytes(void *data, long length)
{
	//fprintf(gfile, "length=%d\n", (int)length);
	out.wait_for_buffer();
	//Sleep(10);
	int remaining = (out.buffer_size * frame_size()) - out.buf_pos;
	if(remaining == length) {
		memcpy(out.wavehdr[out.cur_buf].lpData+out.buf_pos, data, length);
		waveOutWrite(hwo, &out.wavehdr[out.cur_buf], sizeof(WAVEHDR));
		out.bufs_mutex.lock();
		--out.free_bufs;
		out.bufs_mutex.unlock();
		out.buf_pos = 0;
		out.cur_buf = (out.cur_buf+1) % out.num_bufs;
	} else if(remaining > length) {
		memcpy(out.wavehdr[out.cur_buf].lpData+out.buf_pos, data, length);
		out.buf_pos += length;
	} else {
		memcpy(out.wavehdr[out.cur_buf].lpData+out.buf_pos, data, remaining);
		waveOutWrite(hwo, &out.wavehdr[out.cur_buf], sizeof(WAVEHDR));
		out.bufs_mutex.lock();
		--out.free_bufs;
		out.bufs_mutex.unlock();
		out.buf_pos = 0;
		out.cur_buf = (out.cur_buf+1) % out.num_bufs;
		write_bytes(((char*)data)+remaining, length-remaining);
	}

	return length; //write(device_no, data, length);
}

long AudioStream::write_frames(void *data, long length)
{
	return write_bytes(data, length/frame_size()) / frame_size();
  //return 0; //write(device_no, data, length * frame_size()) / frame_size();
}

long AudioStream::wait_bytes(long length)
{
  return 0;
}

long AudioStream::wait_frames(long length)
{
    return 0; //wait_bytes(length * frame_size());
}
